<div>
	<h3 title="mini-buildd commands">Commands</h3>
	<select id="mbd-command" style="width:70%;">
		{% for c in COMMANDS|sort %}  {# 'sort' needed for reproducibility #}
			<option value="{{ c }}" {% if c == "mini-buildd-api" %}selected{% endif %}>{{ c }}</option>
		{% endfor %}
	</select>
	<input type="button" onclick="location.href = '_static/man/' + document.getElementById('mbd-command').value + '.html'" value="Man">
</div>
