<div>
	<h3 title="mini-buildd API calls"><a href="/mini_buildd/api/">API calls</a></h3>
	<form class="search" action="/mini_buildd/api/" method="get" target="_parent">
		<select name="call" style="width:70%;">
			{% for a in APICALLS|sort %}  {# 'sort' needed for reproducibility #}
				<option value="{{ a }}" {% if a == "ls" %}selected{% endif %}>{{ a }}</option>
			{% endfor %}
		</select>
		<input type="submit" value="Doc">
	</form>
</div>
