# Generated by Django 4.2.9 on 2024-01-06 18:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0012_migrate_layout_extra_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='layout',
            name='meta_distributions',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='source',
            name='extra_identifiers',
            field=models.JSONField(blank=True, default=dict),
        ),
    ]
