# Generated by Django 4.2.9 on 2024-01-06 14:14

from django.db import migrations, models
import mini_buildd.models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0008_remove_daemon_hostname'),
    ]

    operations = [
        migrations.AddField(
            model_name='source',
            name='check_valid_until',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='source',
            name='extra_identifiers',
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name='source',
            name='remove_from_component',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='source',
            name='extra_options',
            field=mini_buildd.models.UnixTextField(blank=True),
        ),
    ]
