# Generated by Django 4.2.8 on 2024-01-03 16:08

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0003_remove_daemon_sbuild_jobs'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='distribution',
            name='apt_allow_unauthenticated',
        ),
    ]
