# Generated by Django 4.2.8 on 2024-01-03 16:22

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0005_remove_daemon_smtp_server'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='daemon',
            name='custom_hooks_directory',
        ),
    ]
