# Generated by Django 4.2.9 on 2024-01-07 20:49

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import mini_buildd.models
import mini_buildd.config
import mini_buildd.util


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('mini_buildd', '0021_migrate_suiteoption_extra_options'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='aptkey',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='architecture',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='architectureoption',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='archive',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='chroot',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='component',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='daemon',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='distribution',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='emailaddress',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='layout',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='prioritysource',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='remote',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='repository',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='source',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='subscription',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='suite',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='suiteoption',
            name='extra_options',
        ),
        migrations.RemoveField(
            model_name='uploader',
            name='extra_options',
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_created',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_expires',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_id',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_long_id',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='key_name',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='aptkey',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='architecture',
            name='name',
            field=models.CharField(max_length=50, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='archive',
            name='ping',
            field=models.FloatField(default=-1.0, editable=False),
        ),
        migrations.AlterField(
            model_name='archive',
            name='url',
            field=models.URLField(default='http://ftp.debian.org/debian/', max_length=512, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='btrfssnapshotchroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='architecture',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.architecture'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='debootstrap_command',
            field=models.CharField(default='/usr/sbin/debootstrap', max_length=255),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='debootstrap_extra_options',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='personality',
            field=models.CharField(blank=True, default='', editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='personality_override',
            field=models.CharField(blank=True, default='', max_length=50),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AlterField(
            model_name='chroot',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='component',
            name='name',
            field=models.CharField(max_length=50, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='allow_emails_to',
            field=models.CharField(default=mini_buildd.config.default_allow_emails_to, max_length=254),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='build_queue_size',
            field=models.IntegerField(default=mini_buildd.util.get_cpus),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='custom_archive_origin',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='email_address',
            field=mini_buildd.models.DaemonEmailField(default=mini_buildd.config.default_email_address, max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='ftpd_bind',
            field=models.CharField(default='tcp:port=8067', max_length=200),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='ftpd_options',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='gnupg_keyserver',
            field=models.CharField(default='keyserver.ubuntu.com', max_length=200),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='gnupg_template',
            field=mini_buildd.models.UnixTextField(default='Key-Type: RSA\nKey-Length: 4096\nExpire-Date: 0'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='identity',
            field=models.CharField(default=mini_buildd.config.default_identity, max_length=50),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='keep_build_results',
            field=models.IntegerField(default=5),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='show_last_builds',
            field=models.IntegerField(default=100),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='show_last_packages',
            field=models.IntegerField(default=500),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='wait_for_build_results',
            field=models.IntegerField(default=5),
        ),
        migrations.AlterField(
            model_name='dirchroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='dirchroot',
            name='union_type',
            field=models.IntegerField(choices=[(0, 'aufs'), (1, 'overlayfs'), (2, 'unionfs'), (3, 'overlay')], default=3),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='add_depends',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='autopkgtest_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='base_source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='build_dep_resolver',
            field=models.IntegerField(choices=[(0, 'apt'), (1, 'aptitude'), (2, 'internal')], default=1),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='chroot_setup_script',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='deb_build_options',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='deb_build_profiles',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='internal_apt_priority',
            field=models.IntegerField(default=1),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_extra_options',
            field=models.CharField(default='--info', max_length=200),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=2),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='piuparts_extra_options',
            field=models.CharField(default='--info', max_length=200),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='piuparts_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='piuparts_root_arg',
            field=models.CharField(default='sudo', max_length=200),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_config_blocks',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_setup_blocks',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuildrc_snippet',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='emailaddress',
            name='address',
            field=models.EmailField(max_length=255, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='emailaddress',
            name='name',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='filechroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='filechroot',
            name='compression',
            field=models.IntegerField(choices=[(0, 'no compression'), (1, 'gzip'), (2, 'bzip2'), (3, 'xz')], default=0),
        ),
        migrations.AlterField(
            model_name='layout',
            name='default_version',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%+1', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='experimental_default_version',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%+0', max_length=30),
        ),
        migrations.AlterField(
            model_name='layout',
            name='experimental_mandatory_version_regex',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%\\+0', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='mandatory_version_regex',
            field=models.CharField(default='~%IDENTITY%%CODEVERSION%\\+[1-9]', max_length=100),
        ),
        migrations.AlterField(
            model_name='layout',
            name='meta_distributions',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='layout',
            name='name',
            field=models.CharField(max_length=100, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='looplvmchroot',
            name='loop_size',
            field=models.IntegerField(default=100),
        ),
        migrations.AlterField(
            model_name='looplvmchroot',
            name='lvmchroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.lvmchroot'),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='chroot_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='mini_buildd.chroot'),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='filesystem',
            field=models.CharField(default='ext2', max_length=10),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='snapshot_size',
            field=models.IntegerField(default=4),
        ),
        migrations.AlterField(
            model_name='lvmchroot',
            name='volume_group',
            field=models.CharField(default='auto', max_length=80),
        ),
        migrations.AlterField(
            model_name='prioritysource',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='prioritysource',
            name='priority',
            field=models.IntegerField(default=1),
        ),
        migrations.AlterField(
            model_name='prioritysource',
            name='source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.source'),
        ),
        migrations.AlterField(
            model_name='remote',
            name='http',
            field=models.CharField(default='tcp:host=YOUR_REMOTE_HOST:port=8066', max_length=255, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_created',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_expires',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_id',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_long_id',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='key_name',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='remote',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='remote',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='remote',
            name='wake_command',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='repository',
            name='allow_unauthenticated_uploads',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='external_home_url',
            field=models.URLField(blank=True),
        ),
        migrations.AlterField(
            model_name='repository',
            name='extra_uploader_keyrings',
            field=mini_buildd.models.UnixTextField(blank=True),
        ),
        migrations.AlterField(
            model_name='repository',
            name='identity',
            field=models.CharField(default='test', max_length=50, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='layout',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mini_buildd.layout'),
        ),
        migrations.AlterField(
            model_name='repository',
            name='notify_changed_by',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='notify_maintainer',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='reprepro_morguedir',
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name='repository',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='source',
            name='check_valid_until',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='source',
            name='codename',
            field=models.CharField(default='sid', max_length=60),
        ),
        migrations.AlterField(
            model_name='source',
            name='codeversion',
            field=models.CharField(blank=True, default='', max_length=50),
        ),
        migrations.AlterField(
            model_name='source',
            name='codeversion_override',
            field=models.CharField(blank=True, default='', editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name='source',
            name='description',
            field=models.CharField(blank=True, default='', editable=False, max_length=100),
        ),
        migrations.AlterField(
            model_name='source',
            name='extra_identifiers',
            field=models.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='source',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='source',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='source',
            name='origin',
            field=models.CharField(default='Debian', max_length=60),
        ),
        migrations.AlterField(
            model_name='source',
            name='remove_from_component',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='source',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='distribution',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='package',
            field=models.CharField(blank=True, max_length=100),
        ),
        migrations.AlterField(
            model_name='subscription',
            name='subscriber',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='suite',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='suite',
            name='name',
            field=models.CharField(max_length=100),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='id',
            field=models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_created',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_expires',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_fingerprint',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_id',
            field=models.CharField(blank=True, default='', max_length=100),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_long_id',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='key_name',
            field=models.CharField(blank=True, default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='last_checked',
            field=models.DateTimeField(default=datetime.datetime(1, 1, 1, 0, 0, tzinfo=datetime.timezone.utc), editable=False),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='status',
            field=models.IntegerField(choices=[(0, 'Removed'), (1, 'Prepared'), (2, 'Active')], default=0, editable=False),
        ),
        migrations.AlterField(
            model_name='uploader',
            name='user',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]
