# Generated by Django 4.2.9 on 2024-02-06 15:50

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0029_alter_source_options'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='suiteoption',
            name='auto_migrate_after',
        ),
    ]
