# Generated by Django 4.2.11 on 2024-04-11 16:59

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mini_buildd", "0030_remove_suiteoption_auto_migrate_after"),
    ]

    operations = [
        migrations.AddField(
            model_name="daemon",
            name="expire_builds_after",
            field=models.DurationField(default=datetime.timedelta(days=5)),
        ),
        migrations.AddField(
            model_name="daemon",
            name="expire_events_after",
            field=models.DurationField(default=datetime.timedelta(days=336)),
        ),
    ]
