# Generated by Django 4.2.9 on 2024-01-07 17:32

from django.db import migrations

from . import mbd_get_extra_option


def copy_extra_options(apps, _schema_editor):
    SuiteOption = apps.get_model("mini_buildd", "SuiteOption")
    for suite_option in SuiteOption.objects.all():
        suite_option.rollback = int(mbd_get_extra_option(suite_option, "Rollback", "0"))
        suite_option.save()


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0020_add_suiteoption_extra_options'),
    ]

    operations = [
        migrations.RunPython(copy_extra_options),
    ]
