{% extends "mini_buildd/base.html" %}
{% block title %}{% block titledup %}Sitemap{% endblock %}{% endblock %}

{% block content %}
	{% for name, uris in mbd.config.URIS.items %}
		{% for key, uri in uris.items %}
			<p>
				<a href="{{ uri }}"><code>{{ uri }}</code></a> <b>{{ name }}</b> ({{ key }})
			</p>
		{% endfor %}
	{% endfor %}
{% endblock %}
