<div class="mbd-only-if-popup mbd-error-close-button">
	<a class="mbd-icon" role="button" onclick="$('#mbd-api-popup').addClass('mbd-hidden');"><img class="mbd-icon" src="{% static "icons/close.svg" %}" alt="Close" title="Close Popup (ESC)"></a>
</div>
<h1 class="mbd-only-if-popup">HTTP {{ rfc7807.status.value }}: {{ rfc7807.status.phrase }}</h1>
<p>
	<small><em>{{ rfc7807.status.description }}</em></small>
</p>
<p>
	<strong class="{% if rfc7807.status.value == 200 %}mbd-notice{% else %}mbd-error{% endif %}">{{ rfc7807.detail }}</strong>
</p>
