# Generated by Django 4.2.9 on 2024-01-11 16:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0023_alter_distribution_chroot_setup_script_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='daemon',
            name='keep_build_results',
        ),
        migrations.RemoveField(
            model_name='daemon',
            name='wait_for_build_results',
        ),
    ]
