# Generated by Django 4.2.8 on 2024-01-03 15:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='aptkey',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='architecture',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='architectureoption',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='archive',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='chroot',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='component',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='daemon',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='distribution',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='emailaddress',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='layout',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='prioritysource',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='remote',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='repository',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='source',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='subscription',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='suite',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='suiteoption',
            name='pickled_data',
        ),
        migrations.RemoveField(
            model_name='uploader',
            name='pickled_data',
        ),
    ]
