{% extends "admin/index.html" %}
{% load i18n %}

{# Should be in-sync with django's 'admin/index.html', with some custom includes sprinkled in (DRY -- but how, django?). #}
{# It *should* be in-sync with version of lowest django upstream we support (as per debian/control). #}

{% block content %}
	{% include "mini_buildd/includes/popup.html" %}

	<div id="content-main">
		{% include "admin/mbd_setup_header.html" %}
		{% include "admin/app_list.html" with app_list=app_list show_changelinks=True %}
	</div>
{% endblock content %}
