{% extends "mini_buildd/base.html" %}
{% block title %}{% block titledup %}Register User{% endblock %}{% endblock %}

{% block content %}
	<form action="/accounts/register/" method="post">
		{% csrf_token %}
		<table>{{ form.as_table }}</table>
		<input type="submit">
	</form>
{% endblock %}
