# Generated by Django 4.2.8 on 2024-01-03 16:15

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0004_remove_distribution_apt_allow_unauthenticated'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='daemon',
            name='smtp_server',
        ),
    ]
