<header class="mbd-grid-lcr">
	<nav class="mbd-result-titlebar-apicall">
		<ul>
			<li>{% mbd_api view.mbd_call %}</li>
			<li class="mbd-success">
				<strong><code>API call succeeded</code></strong>
			</li>
		</ul>
	</nav>
	<nav class="mbd-result-titlebar-ucj">
		<ul>
			<li>
				{% mbd_absolute_uri view.mbd_call.url_relative as url_abs %}
				<details class="mbd-only-if-popup">
					<summary>URL</summary>
					<section>
						<a class="mbd-underline" href="{{ view.mbd_call.url_relative }}" title="Only works for unconfirmable calls">Call again (as non-popup)</a>
						{% mbd_file content=url_abs title="URL" %}
					</section>
				</details>
			</li>
			<li><details><summary>CLI</summary><section>{% mbd_file content=view.mbd_call.command_line_full title="CLI" %}</section></details></li>
			<li><details><summary>JSON</summary><section>{% mbd_file content=view.mbd_call.json_pretty title="JSON" %}</section></details></li>
		</ul>
	</nav>
	<nav class="mbd-result-titlebar-buttons">
		<ul>
			<li>
				<a href="/" target="_parent">
					<img class="mbd-icon" src="{% static 'icons/home.svg' %}" alt="Home" title="Home: {{ mbd.daemon.get_title }}">
				</a>
			</li>
			<li>
				<a role="button" onclick="location.reload();">
					<img class="mbd-icon" src="{% static "icons/refresh.svg" %}" alt="Reload" title="Reload Parent Page (F5)">
				</a>
			</li>
			<li class="mbd-only-if-popup">
				<a role="button" onclick="$('#mbd-api-popup').addClass('mbd-hidden');">
					<img class="mbd-icon" src="{% static "icons/close.svg" %}" alt="Close" title="Close Popup (ESC)">
				</a>
			</li>
		</ul>
	</nav>
</header>

{% include "mini_buildd/includes/messages.html" %}

<section>
	{% block results %}{% endblock %}
</section>
