# Generated by Django 4.2.8 on 2024-01-03 16:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0006_remove_daemon_custom_hooks_directory'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='aptkey',
            name='auto_reactivate',
        ),
        migrations.RemoveField(
            model_name='chroot',
            name='auto_reactivate',
        ),
        migrations.RemoveField(
            model_name='daemon',
            name='auto_reactivate',
        ),
        migrations.RemoveField(
            model_name='remote',
            name='auto_reactivate',
        ),
        migrations.RemoveField(
            model_name='repository',
            name='auto_reactivate',
        ),
        migrations.RemoveField(
            model_name='source',
            name='auto_reactivate',
        ),
        migrations.RemoveField(
            model_name='uploader',
            name='auto_reactivate',
        ),
    ]
