{% extends "admin/base.html" %}

{% block extrahead %}
	{{ block.super }}
	{% include "mini_buildd/includes/head.html" %}

	<link rel="stylesheet" type="text/css" href="{% static 'admin.css' %}?v={{ mbd.VERSIONS.version }}">

	<!-- start with light theme turned on -->
	<script>localStorage.setItem("theme", "light");</script>
{% endblock %}

{% block branding %}
	<a href="/" target="_parent">
		<img class="mbd-icon" src="{% static 'icons/mini-buildd.svg' %}" alt="Home" title="Home: {{ mbd.daemon.get_title }}">
	</a>
{% endblock %}
