{% if model.mbd %}
	<td>
		{% mbd_model_stats model.mbd.class_path as stats %}
		(<span title="Total instances"><b>{{ stats.total }}</b></span>
		{% if stats.active %}<span title="Active instances" class="mbd-model-status-Active">&nbsp;<b>{{ stats.active }}</b></span>{% endif %}
		{% if stats.prepared %}<span title="Prepared instances" class="mbd-model-status-Prepared">&nbsp;<b>{{ stats.prepared }}</b></span>{% endif %}
		{% if stats.removed %}<span title="Removed instances" class="mbd-model-status-Removed">&nbsp;<b>{{ stats.removed }}</b></span>{% endif %})
	</td>
{% endif %}
