<script>
	$("#{{ identity }}").hide();  // Avoid table "flickering" (see "initComplete" below)
	$(document).ready(function() {$("#{{ identity }}").
																	DataTable({dom: "fit",
																						 paging: false,
																						 language: {"search": "", "searchPlaceholder": "Start typing to filter..."},
																						 {% if order %}order: [{{ order }}]{% else %}ordering: false{% endif %},
																						 columns: {{ columns }},
																						 initComplete: function()
																		{
																			$("#{{ identity }}").addClass("cell-border compact stripe");
																			$("#{{ identity }}").show();
																			$("#{{ identity }}").DataTable().columns.adjust();
																		},
																	});});
</script>
