# Generated by Django 4.2.9 on 2024-01-11 16:43

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0024_remove_daemon_keep_build_results_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='daemon',
            name='show_last_builds',
        ),
    ]
