// Project config
#include "config.h"

// Implementation
#include "Lib.hpp"

// STDC++
#include <iostream>

namespace MBD_TEST_FTBFS
{
	void mbd_test_ftbfs()
	{
		std::cout << "Test project mbd-test-ftbfs." << std::endl;
	}
}
