# Generated by Django 4.2.9 on 2024-01-07 17:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0019_migrate_chroot_extra_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='suiteoption',
            name='rollback',
            field=models.IntegerField(default=0),
        ),
    ]
