# Generated by Django 4.2.9 on 2024-01-29 15:44

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0025_remove_daemon_show_last_builds'),
    ]

    operations = [
        migrations.RenameField(
            model_name='daemon',
            old_name='show_last_packages',
            new_name='event_queue_size',
        ),
        migrations.RenameField(
            model_name='daemon',
            old_name='build_queue_size',
            new_name='max_parallel_builds',
        ),
        migrations.RenameField(
            model_name='source',
            old_name='codeversion_override',
            new_name='codeversion_guessed',
        ),
    ]
