# Generated by Django 4.2.9 on 2024-02-06 13:02

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0028_migrate_source_codeversion_override'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='source',
            options={},
        ),
    ]
