{% extends "mini_buildd/base.html" %}
{% block title %}{% block titledup %}HTTP {{ rfc7807.status.value }}: {{ rfc7807.status.phrase }}{% endblock %}{% endblock %}

{% block content %}
	{% include "mini_buildd/includes/error.html" %}
{% endblock %}
