<meta http-equiv="content-type" content="text/html; charset=utf-8">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<meta name="Description" content="mini-buildd ({{ mbd.daemon.get_title }})">
<link rel="icon" type="image/svg+xml" href="{% static 'icons/mini-buildd.svg' %}?v={{ mbd.VERSIONS.version }}">

<link rel="stylesheet" type="text/css" href="{% static 'javascript/jquery-datatables/css/jquery.dataTables.min.css' %}">
<link rel="stylesheet" type="text/css" href="{% static 'mini_buildd.css' %}?v={{ mbd.VERSIONS.version }}">

<!-- JavaScript Setup -->
<script>
	// On js error, display alert. Otherwise functionality might be silently missing for joe netizen.
	window.onerror = function(msg, url, line, col, err)
	{
		alert("JavaScript error detected. Please debug, patch and report ;). Or consider disabling JavaScript: " + err);
	};
</script>
<script src="{% static 'javascript/jquery/jquery.min.js' %}"></script>
<script src="{% static 'javascript/jquery-datatables/jquery.dataTables.min.js' %}"></script>

<script src="{% static 'javascript/clipboard/clipboard.js' %}"></script>
<!-- Compat path for node-clipboard < 2.0.11 -->
<script>if (typeof ClipboardJS == 'undefined') { document.write(unescape("%3Cscript src='{% static 'javascript/clipboard.js/clipboard.js' %}' type='text/javascript'%3E%3C/script%3E")); }</script>

<script src="{% static 'mini_buildd.js' %}?v={{ mbd.VERSIONS.version }}"></script>
