.root-code
{
	background-color: thistle;
}
.root-code::before
{
	content: "Run code as user root";
	font-style: italic;
	filter: opacity(30%);
}
.mini-buildd-code
{
	background-color: lightblue;
}
.mini-buildd-code::before
{
	content: "Run code as user mini-buildd";
	font-style: italic;
	filter: opacity(30%);
}
img.logo-menu
{
	vertical-align: middle;
	height: 1.4em;
}
/* 1st line of an admonition */
p.admonition-title + p
{
	font-weight: bolder;
}

/* finetune current theme ("Alabaster") */
a.reference.internal span.pre { white-space: normal; }
body { background-color: #f7f7f7; }
body>div.document { width: 98%; }
div.body[role="main"] { border-radius: .4rem; border: .1rem solid #ccc; padding: 1rem 2rem; }
div.bodywrapper { margin-left: 16rem; }
div.sphinxsidebar #searchbox input[type="submit"] { margin-left: .2rem; border-left: 1px solid #ccc; padding: .2rem; }
div.sphinxsidebar #searchbox input[type="text"] { width: 71%; padding: .2rem .5rem; }
div.sphinxsidebar h3 { margin-top: 1.6rem; }
div.sphinxsidebar input { padding: .2rem .5rem; border-radius: .3rem; }
div.sphinxsidebar { width: 16rem; font-size: 1rem; }
div.sphinxsidebar ul li.toctree-l1 > a { font-size: 110%; }
div.sphinxsidebar ul li.toctree-l2 > a { font-size: initial; }
div.sphinxsidebarwrapper p.logo { margin: 0 0 2rem -1.2rem; }
div.sphinxsidebarwrapper select { font-size: 1rem; padding: .1rem .5rem; }
div.sphinxsidebarwrapper { padding: .5rem .5rem 1rem 0; }

/* alabaster theme special (870px) */
@media screen and (max-width: 870px)
{
	body { padding: .2rem; }
	body>div.document { width: initial; }
	div.bodywrapper { margin-left: 0; }
	div.sphinxsidebar { float: none; width: fit-content; color: #fff; margin: 0; border-radius: .4rem; }
	img.logo { margin-top: 2rem; }
}
