# Generated by Django 4.2.8 on 2024-01-03 16:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0007_remove_auto_reactivate'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='daemon',
            name='hostname',
        ),
    ]
