<span class="mbd-sbuild-status-{{ bres.sbuild_status }}" title="sbuild: {{ bres.sbuild_status }}">B</span>
{% for check, status in checks.items %}
	<span class="mbd-sbuild-check-{{ status }}" title="{{ check }}: {{ status }}">{{ check.0.upper }}</span>
{% endfor %}
{% if bres.canceled_by %}
- <span class="mbd-notice" title="by {{ bres.canceled_by }}"><strong>Canceled</strong></span>
{% endif %}
