# Generated by Django 4.2.9 on 2024-02-01 14:38

from django.db import migrations


# Keep old override behaviour for existing instances.
# Previously, codeversion in essence behaved like codeversion_override
def migrate_source_codeversion_override(apps, _schema_editor):
    Source = apps.get_model("mini_buildd", "Source")
    for source in Source.objects.all():
        source.codeversion_override = source.codeversion
        source.save()


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0027_source_codeversion_override'),
    ]

    operations = [
        migrations.RunPython(migrate_source_codeversion_override),
    ]
