<header class="mbd-grid-lcr">
	<h2>Source: {{ view.mbd_call.args.source.value }}</h2>
	<span></span>
	<nav>
		<ul>
			<li><a class="mbd-icon" href="https://tracker.debian.org/pkg/{{ view.mbd_call.args.source.value }}"><img class="mbd-icon" src="{% static 'icons/debian-pts.svg' %}" alt="Debian PTS" title="Show source package in the Debian Package Tracker"></a></li>
			<li class="mbd-only-if-popup"><a class="mbd-icon" href="{{ view.mbd_call.url_relative }}" target="_parent" title="Call and show as non-popup"><img class="mbd-icon" src="{% static 'icons/isolate.svg' %}" alt="Isolate"></a></li>
			<li>{% mbd_api "subscribe" value_source=view.mbd_call.args.source.value %}</li>
		</ul>
	</nav>
</header>
