# Generated by Django 4.2.9 on 2024-01-06 17:37

from django.db import migrations, models
import mini_buildd.models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0010_migrate_source_extra_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='layout',
            name='meta_distributions',
            field=models.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name='layout',
            name='extra_options',
            field=mini_buildd.models.UnixTextField(blank=True),
        ),
    ]
