# Generated by Django 4.2.9 on 2024-01-07 16:09

from django.db import migrations, models
import mini_buildd.config


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0015_migrate_distribution_extra_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='daemon',
            name='custom_archive_origin',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
        migrations.AlterField(
            model_name='daemon',
            name='identity',
            field=models.CharField(default=mini_buildd.config.default_identity, max_length=50),
        ),
    ]
