{% extends "mini_buildd/base.html" %}
{% block title %}{% block titledup %}{{ view.mbd_title }}{% endblock %}{% endblock %}
{% block menu %}<li><a class="mbd-icon" href="{{ view.mbd_uri }}"><img class="mbd-icon" src="{% static 'icons/standalone-view.svg' %}" alt="Standalone View" title="Standalone View"></a></li>{% endblock %}

{% block content %}
	<iframe title="{{ view.mbd_title }}" src="{{ view.mbd_uri }}"></iframe>
{% endblock %}
