# Generated by Django 4.2.9 on 2024-02-01 14:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0026_rename_show_last_packages_daemon_event_queue_size_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='source',
            name='codeversion_override',
            field=models.CharField(blank=True, default='', max_length=50),
        ),
    ]
