select              { height: initial; }      /* admin css: fixed pixels */
select[multiple]    { min-height: initial; }  /* admin css: fixed pixels */
.dashboard #content { width: 70%; }           /* admin css: fixed pixels (600px) */
:root { --bg-color-pre: rgba(0, 0, 0, .05); } /* make preformatted text distinguishable in "dark mode" */
body, li, dt, dd { font-size: initial; }
input[type="checkbox"] { margin-right: .2rem; }

/* Login */
body.login { background-color: white; }
body.login img.mbd-icon { height: 3.6rem; }
body.login #container { background-color: #fafafa; border-radius: .5rem; border: .1rem solid var(--color-border-header); }
body.login #header { background-color: var(--bg-header); padding: 0.5rem; margin: 0; border-bottom: .1rem solid var(--color-border-header); }
body.login #main { padding: .2rem .4rem .2rem .4rem; }
body.login #content-start, body.login div.content { background-color: transparent; border: 0; margin: 0 0 -.2rem 0; }
body.login #content { padding: .5rem 1.4rem 0 1rem; }
body.login #login-form{ display: block; background-color: #f3f3f3; border-color: transparent; margin-bottom: -.3rem; }
body.login #login-form div.submit-row { background-color: transparent; padding: .6rem 0 0 .6rem; }
body.login input[type="submit"]:hover { background-color: #056f8f; }
body.login input[type="submit"] { font-size: 1rem; font-weight: bold; border: .1rem solid #05536b; border-radius: .4rem; background-color: var(--bg-color-submit); padding: .4rem 1rem; margin: .4rem .2rem .2rem .2rem; }

/* Setup */
#header { background-color: #057b9e; border-radius: .4rem .4rem 0 0; margin: .8rem .5rem 0 .5rem; }
#content h1 { margin-bottom: .5rem; }
#content-start { background-color: #f3f3f3; border-radius: .4rem; border: .1rem solid var(--color-border-section-etc); margin: 0 .5rem 1rem 1rem; }
#main > #nav-sidebar + #content-start { margin-top: -.5rem; }
#main > div.content { background-color: #f3f3f3; border-radius: .4rem; border: .1rem solid var(--color-border-section-etc); margin: .8rem .5rem .8rem .5rem; } /* bookworm */
#main > #nav-sidebar + div.content { margin-top: -.5rem; } /* bookworm */
#content-main>form, form#changelist-form { background-color: #fcfcfc;  border-color: var(--color-border-section-etc); }
#content-related { background-color: transparent; }
#changelist.module { background-color: transparent; }
#changelist-filter { background-color: #fcfcfc; border-radius: .4rem;  border: .1rem solid var(--color-border-section-etc); }
#changelist-filter h2 { border-radius: .4rem .4rem 0 0; }
#toolbar { background-color: #f3f3f3; }
#toggle-nav-sidebar { background-color: transparent; font-size: 1.6rem; border: 0; }
#change-history p.paginator, p.paginator { border: 0; background-color: transparent; }
nav#nav-sidebar { background-color: transparent; border: 0; }
.main > #nav-sidebar { display: none; margin-top: -1rem; } /* bookworm */
.main.shifted > #nav-sidebar { display: block; } /* bookworm */
nav#nav-sidebar caption { border-radius: .4rem .4rem 0 0; background-color: #057b9e; }
nav#nav-sidebar h2 { background-color: #056f8f; border-radius: .3rem; }
input#nav-filter { background-color: white; border-radius: .4rem; margin: .5rem 0 0 0; padding: .3rem .5rem; border: .1rem solid var(--color-border-section-etc); }
form#changelist-form .actions, form#changelist-form .results, form#changelist-form .paginator {background-color: transparent; border: 0; }
form fieldset.module.aligned { background-color: transparent; }
div.app-mini_buildd.module, div.app-auth.module { background-color: #eee; padding: 1rem; border-radius: .5rem; border: .1rem solid var(--color-border-section-etc); }
div.app-mini_buildd.module { margin: .5rem 0 -.5rem 0; }
div.app-auth.module:last-child { margin-bottom: .8rem; }
div.app-auth.module { margin: 1rem 0 -1rem 0; }
div.breadcrumbs { background-color: #056f8f; border-radius: 0 0 .4rem .4rem; margin: 0 .5rem 1rem .5rem; padding: .4rem 2.2rem; }
div.actions button { font-weight: bold; border-radius: .3rem; margin: 0 .3rem .3rem 0; padding: .2rem .8rem; border: .1rem solid #bbb; }
div.actions button:hover { background-color: #f7f7f7; }
div.actions button:active { border-color: #eee; }
div.actions button:focus { animation: mbd_ongoing 1s infinite alternate-reverse; }
div.submit-row { background-color: var(--bg-color-1); }
body { background-color: #f9f9f9; padding: 0 .5rem; }
body.dashboard { background-color: #f9f9f9; }
body.dashboard h1 {font-weight: bold; margin-bottom: .5rem; }
body.dashboard form.mbd-api>details[open] { margin-top: -.26rem; }
body.dashboard form.mbd-api>details[open]>summary { margin: -.1rem 0 .2rem -.05rem; }
body.dashboard #header { background-color: #057b9e; border-radius: .4rem; margin: .8rem .5rem 0 .5rem; }
body.dashboard #header .mbd-icon, body.app-mini_buildd #header .mbd-icon, body.app-auth #header .mbd-icon { height: 2.4rem; margin: -.3rem -.2rem -.3rem -.2rem;  }
body.dashboard #content {  margin: -.5rem 0 -.5rem 0; }
body.dashboard #content-start { margin: .8rem .5rem .8rem .5rem; }
body.dashboard #content-main>section { background-color: var(--bg-color-2); }
body.dashboard #content-main table>caption { background-color: #057b9e; border-radius: .4rem .4rem 0 0; }
body.dashboard #content-main td>h2 { background-color: #056f8f; border-radius: .3rem; }
body.dashboard #recent-actions-module { background-color: #fbfbfb; border-radius: .4rem;  border: .1rem solid var(--color-border-section-etc); }
body.dashboard #recent-actions-module h2 { background-color: #057b9e; color: white; border-radius: .4rem .4rem 0 0; }
body.dashboard.app-mini_buildd #header, body.dashboard.app-auth #header { border-radius: .4rem .4rem 0 0; }
ul.messagelist li { border-radius: .4rem; }
.object-tools a { border-radius: .4rem; }
.object-tools a:link, .object-tools a:visited { font-weight: bold; font-size: .75rem; border: 0.1rem solid #05536b; background-color: var(--bg-color-submit); }
.object-tools a:focus, .object-tools a:hover { background-color: #056f8f; }
.vTextField { min-width: 20em; width: 42%; }

/* medium small screen */
@media screen and (max-width: 75rem)
{
	#content>#content-related { width: 100%; margin: 0; }
}

/* django admin special (1024px, 767px) */
@media screen and (max-width: 1024px)
{
	#changelist-search span.small.quiet { width: 3.6rem; }
	#content-main #toolbar form #searchbar { width: 7rem; }
	body.app-mini_buildd #header .mbd-icon { height: 3rem; margin-bottom: .3rem; }
	body.dashboard #header .mbd-icon { height: 3rem; margin-bottom: .2rem; }
}
@media screen and (max-width: 767px)
{
	#main>#nav-sidebar { display: none; }
	body.login #content { margin-bottom: .7rem; }
}

/* small screen (fits/overwrites jquery data) */
@media screen and (max-width: 640px)
{
	#content-main>form, form#changelist-form { margin: 0; }
	#content-start>#content { padding: .5rem .2rem; margin-bottom: -.3rem; }
	#header { margin: 0; }
	#main>div.content { margin: .4rem 0 0 0; }
	#main>div.content, #main>div.content>#content { padding: .5rem .2rem; margin-bottom: -.3rem; } /* bookworm */
	.main.shifted>#nav-sidebar { display: none; }
	.mbd-grid-lcr { grid-template-columns: repeat(auto-fit, minmax(15rem, auto)); }
	body { padding: 0; }
	body, li, dt, dd { font-size: .9rem; }
	body.dashboard #header { margin-left: 0; margin-right: 0; margin-top: 0; }
	body.login #content { padding-bottom: .8rem; }
	body.login #login-form { margin: 0; }
	body.login #main { margin-bottom: .7rem; }
	div.app-mini_buildd.module, div.app-auth.module { padding: .32rem; }
	div.breadcrumbs { margin-left: 0; margin-right: 0; }
	input, textarea, select, .form-row p, form .button { font-size: .75rem; }
	section { padding: .8rem .5rem; }
	tr td, tr th { font-size: .8rem; }
}
