{% if messages %}
	<aside>
		<details>
			<summary><strong>{{ messages|length }} Message(s)</strong></summary>
			<ul class="mbd-messagelist">
				{% for message in messages %}
					<li class="mbd-django-message-{{ message.tags }}">{{ message|capfirst }}</li>
				{% endfor %}
			</ul>
		</details>
	</aside>
{% endif %}
