{% extends "mini_buildd/includes/api/base.html" %}
{% block results %}
	{% mbd_get view.mbd_call.args.source.value as source %}  {# Shortcut #}
	{% include "mini_buildd/includes/api/source_header.html" %}
	{% for repository, results in view.mbd_call.result.items %}
		<section>
			<h3>Repository {{ repository }}</h3>
			<ul>
				<li>{% mbd_api "show" name="all" value_source=source value_repositories=repository %}</li>
				{% for codename in results.codenames %}
					<li>{% mbd_api "show" name=codename value_source=source value_repositories=repository value_codenames=codename.codename %}</li>
				{% endfor %}
			</ul>
		</section>
	{% endfor %}
{% endblock %}
