# Generated by Django 4.2.9 on 2024-01-07 16:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0017_migrate_daemon_extra_options'),
    ]

    operations = [
        migrations.AddField(
            model_name='chroot',
            name='debootstrap_command',
            field=models.CharField(default='/usr/sbin/debootstrap', max_length=255),
        ),
        migrations.AddField(
            model_name='chroot',
            name='debootstrap_extra_options',
            field=models.CharField(blank=True, default='', max_length=255),
        ),
    ]
