# Generated by Django 4.2.14 on 2024-07-24 15:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0031_daemon_expire_builds_after_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='distribution',
            name='lintian_warnfail_options',
            field=models.CharField(default='--fail-on error,warning', max_length=512),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_extra_options',
            field=models.CharField(default='--info --suppress-tags bad-distribution-in-changes-file,bogus-mail-host,bogus-mail-host-in-debian-changelog', max_length=512),
        ),
    ]
