{% mbd_token as token %}

<header class="mbd-grid-lcr mbd-file">
	<strong>{% if uri %}File <code>{{ uri|mbd_basename }}</code>{% if mtime %} (<span class="mbd-localize-date" title="Modified">{{ mtime|mbd_fromtimestamp|date:"r" }}</span>){% endif %}{% else %}{{ title|default:"Data" }}{% endif %}</strong>
	<span></span>
	<nav>
		<ul class="mbd-clipboard-download-buttons">
			{% if content is not None %}
				<li>
					<a class="mbd-icon mbd-clipboard" data-clipboard-target="#{{ token }}">
						<img class="mbd-icon" src="{% static 'icons/copy.svg' %}" alt="Copy to clipboard" title="Copy to clipboard">
					</a>
				</li>
			{% endif %}
			{% if uri %}
				<li><a class="mbd-icon" href="{{ uri }}" download><img class="mbd-icon" src="{% static 'icons/download.svg' %}" alt="Download" title="{{ uri }}"></li></a>
			{% endif %}
		</ul>
	</nav>
</header>

{% if toc %}
	<details class="mbd-file-toc" open>
		<summary>Table Of Contents</summary>
		<ul>
			{% for entry in toc %}
				<li><a class="{{ entry.class }}" href="#{{ entry.token }}">{{ entry.title }}</a></li>
			{% endfor %}
		</ul>
	</details>
{% endif %}

{% if content is None %}
	<div class="mbd-info">No filter -- content will not be displayed.</div>
{% else %}
	<pre id="{{ token }}" class="mbd-file-content">{{ content }}</pre>
{% endif %}
