# Generated by Django 4.2.9 on 2024-01-08 09:39

from django.db import migrations, models
import mini_buildd.models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0022_remove_extra_options'),
    ]

    operations = [
        migrations.AlterField(
            model_name='distribution',
            name='chroot_setup_script',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuildrc_snippet',
            field=mini_buildd.models.UnixTextField(blank=True, default=''),
        ),
        migrations.AlterField(
            model_name='source',
            name='codename',
            field=models.CharField(default='sid', max_length=60),
        ),
    ]
