<section>
	<div class="mbd-warning">
		{% if mbd.daemon.is_alive %}<section><strong>Daemon running (accepting incoming)</strong>: You may want to <em>stop</em> beforehand (each <code>setup</code> calls will implicitly run stopped, discarding ongoing events every time)</section>{% endif %}
	</div>

	<header class="mbd-grid-lcr">
		<nav>
			<ul>
				<li><a class="mbd-icon" href="{% url 'manual-static' %}administrator.html#setup" title="Jump to Setup section of Administrator's Manual"><img class="mbd-icon" src="{% static 'icons/manual.svg' %}" alt="Manual"></a>
					<li>{% mbd_api mbd.api.Setup.setup_inspect name="Inspect" %}</li>
					<li>{% mbd_api mbd.api.Setup.setup_extend name="Extend" %}</li>
			</ul>
		</nav>
		<nav>
			<ul>
				<li>{% mbd_api "setup" %}</li>
			</ul>
		</nav>
		<nav>
			<ul>
				{% for name, setup in mbd.api.SETUP_BOOTSTRAP.items %}
					<li>
						<img class="mbd-icon" src="{% static 'icons/'|add:name|lower|add:'.svg' %}" alt="{{ name }}">
						{% mbd_api setup name="Bootstrap "|add:name %}
					</li>
				{% endfor %}
			</ul>
		</nav>
	</header>
</section>
