# Generated by Django 4.2.9 on 2024-01-07 14:28

from django.db import migrations, models
import mini_buildd.models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0013_alter_layout_meta_distributions_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='distribution',
            name='add_depends',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='distribution',
            name='autopkgtest_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=0),
        ),
        migrations.AddField(
            model_name='distribution',
            name='deb_build_options',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='distribution',
            name='deb_build_profiles',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='distribution',
            name='internal_apt_priority',
            field=models.IntegerField(default=1),
        ),
        migrations.AddField(
            model_name='distribution',
            name='sbuild_config_blocks',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='distribution',
            name='sbuild_setup_blocks',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='extra_options',
            field=mini_buildd.models.UnixTextField(blank=True),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='lintian_mode',
            field=models.IntegerField(choices=[(0, 'DISABLED'), (1, 'IGNORE'), (2, 'ERRFAIL'), (3, 'WARNFAIL')], default=2),
        ),
    ]
