# Generated by Django 4.2.14 on 2024-08-03 12:48

from django.db import migrations, models
import mini_buildd.models


class Migration(migrations.Migration):

    dependencies = [
        ('mini_buildd', '0034_sync_help_text_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_config_blocks',
            field=models.CharField(blank=True, default='set-default-path ccache', help_text='<pre>Ordered sbuild config blocks (space-separated)\n\nDefault: set-default-path ccache\n\nAvailable blocks:\n\nset-default-path: [default] Explicitly set $path (to sbuild&#x27;s default value), so later config blocks may add to it\nccache          : [default] Update $path and $$build environment to support ccache</pre>', max_length=255, validators=[mini_buildd.models.SbuildConfigBlocksValidator()]),
        ),
        migrations.AlterField(
            model_name='distribution',
            name='sbuild_setup_blocks',
            field=models.CharField(blank=True, default='apt-setup apt-update ccache eatmydata', help_text='<pre>Ordered sbuild setup blocks (space-separated)\n\nDefault: apt-setup apt-update ccache eatmydata\n\nAvailable blocks:\n\napt-setup                : [default] Sets up APT for buildrequest (APT keys, optional https, sources and preferences)\napt-update               : [default] Update APT\nccache                   : [default] Installs ccache (together with the &#x27;ccache&#x27; config block, this enables ccache for builds on the same instance)\neatmydata                : [default] Install eatmydata and activate via LD_PRELOAD when possible (may improve speed)\nshow-sbuildrc            : [extra] Print generated &#x27;.sbuildrc&#x27; (informational only)\napt-key-binary           : [extra] Convert APT keys to binary (needed for Debian &lt;= jessie, Ubuntu &lt;= yakketti) -- place between ``apt-setup`` and ``apt-update``\napt-key-add              : [extra] Add APT keys via deprecated ``apt-key add`` (needed for Debian &lt;= squeeze) -- place between ``apt-setup`` and ``apt-update``\napt-no-check-valid-until : [extra] Set APT option &quot;Acquire::Check-Valid-Until false&quot; (needed when Release files are expired) -- place before ``apt-setup``\napt-allow-unauthenticated: [extra] Set APT option &quot;APT::Get::AllowUnauthenticated true&quot; (needed when APT keys are expired) -- place before ``apt-setup``\napt-force-yes            : [extra] Set APT option &quot;APT::Get::Force-Yes true&quot; (compat(urold)) -- place before ``apt-setup``\napt-no-cache             : [extra] Set APT option &quot;Acquire::http::No-Cache true&quot; (compat(urold)) -- place before ``apt-setup``\napt-no-pipelining        : [extra] Set APT option &quot;Acquire::http::Pipeline-Depth 0&quot; (compat(urold)) -- place before ``apt-setup``\napt-clear                : [extra] Try to clear any redundant APT data in ``/var/`` (experimental: can make APT work again if stuck)\nbash-devices             : [extra] Fixup /dev/fd|stdin|stdout|stderr. Needed for *building* bash only. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=327477 (compat(bash &lt; 3.0-17 (2005))\nschroot-shm              : [extra] Fixup /dev/shm mount. See https://bugs.debian.org/cgi-bin/bugreport.cgi?bug=728096 (compat(schroot &lt; 1.6.10-3))\nsun-java6-license        : [extra] Accept sun-java6 licence (debconf) so we can build-depend on it (sun-java is no longer part of Debian since 2011)</pre>', max_length=255, validators=[mini_buildd.models.SbuildSetupBlocksValidator()]),
        ),
    ]
